#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""TMNAS Event handler."""

from __future__ import with_statement

import os
import sys
import re
import signal

import iobase.xmldata as xmldata
import ionas.request as request

from fal_info import (FAL_EVENTLOG_PATH, FAL_BUZZER_PATH, BUZZER_ERROR,
                        Results, ReqTypes)
from fal_msginfo import (Msg,)


EVENTLOG_BODY_TMPL = "<eventlog><log><code>%s</code></log></eventlog>"
LOG_NAME = "log"
ARGLIST_NAME = "arglist"
ARG_NAME = "arg"
NAME_NAME = "name"
NAME_ATTR_NAME = "name"

SHARE_NAME = "share"
FILE_NAME = "file"

MNT_POINT = os.path.join(os.sep, "mnt")
INTARNAL_VOL_BASE = os.path.join(MNT_POINT, "sataraid1", "share")
EXT_SHARE_LIST = ("usb1", "usb2", "usb3", "usb4", "usb5")

BUZZER_BODY_TMPL = '<buzzer><melody type="%s"/></buzzer>'

ARCHIVE_PATH_RE = "(.*[^\s])\s*\(.*\)"
PATH_IN_ARCHIVE_RE = ".*[^\s]\s*\((.*)\)"


NOTIFY_FILE_DELETED_PREFIX = "ウイルスが検出されたため削除されました_"
NOTIFY_FILE_QUARANTINED_PREFIX = "ウイルスが検出されたため隔離されました_"
NOTIFY_FILE_UNHANDLED_PREFIX = "ウイルスが検出され適切に処理できませんでした_"
SPOOL_PATH_ENV = "TMNAS_EVENT_SPOOL_PATH"

NOTIFICATION_NAME = "notification"
FILE_NAME_NAME = "file-name"
SCAN_ACTION_RESULT_NAME = "scan-action-result"
TYPE_ATTR = "type"

VIRUSINFECTION_TYPE = "VirusInfection"

RESULT_DELETED = "Deleted."
RESULT_QUARANTINED = "Quarantined."
RESULT_CLEANED = "Cleaned."
RESULT_RENAMED = "Renamed."
RESULT_PASSED = "Passed."


def __send_request(target, entry):
    """Send request entry to "target"."""

    return request.Request(target).request(entry).get_entry()


def __get_real_dir(path):
    """Return a real directory for "path"."""

    dir_path = os.path.dirname(path)
    if dir_path[0]:
        dir_path = dir_path[1:]

    for ext_share in EXT_SHARE_LIST:
        if dir_path.startswith(ext_share):
            return os.path.join(MNT_POINT, dir_path)

    return os.path.join(INTARNAL_VOL_BASE, dir_path)


def __create_notify_file(target_path, prefix):
    """Create notify file for Virus."""

    with open(os.path.join(__get_real_dir(target_path),
            prefix + os.path.basename(target_path)), "w"):
        pass


def __try_get_path_in_archive(notification_node):
    """Return a path in archive file."""

    match = re.match(PATH_IN_ARCHIVE_RE,
                        notification_node.get_value(FILE_NAME_NAME))
    if match:
        return match.group(1)
    return ""

def __get_target_path(notification_node):
    """Return a target file path in notification_node."""

    target_path = notification_node.get_value(FILE_NAME_NAME)
    match = re.match(ARCHIVE_PATH_RE, target_path)
    if match:
        return match.group(1)

    return target_path

def __get_share_name(path):
    """Return a share name for "path"."""

    split_path_list = path.split(os.sep)
    if split_path_list[0]:
        return split_path_list[0]

    return split_path_list[1]


def reset_signal():
    """Reset signal to default."""

    signal.signal(signal.SIGCHLD, signal.SIG_DFL)
    signal.signal(signal.SIGPIPE, signal.SIG_DFL)


def error_buzzer():
    """Issue a buzzer for the error."""

    entry = request.Entry(ReqTypes.PLAY, BUZZER_BODY_TMPL % (BUZZER_ERROR,))
    __send_request(FAL_BUZZER_PATH, entry)


def log_eventlog(code, arg_dict={}):
    """Log to eventlog."""

    entry = request.Entry(ReqTypes.ADD, EVENTLOG_BODY_TMPL % (code,))
    if len(arg_dict):
        arglist = xmldata.XmlNode(ARGLIST_NAME)
        for arg in arg_dict:
            arglist.append_child(ARG_NAME, arg_dict[arg], {NAME_ATTR_NAME: arg})
            entry.get().get_node(LOG_NAME).append_child(arglist)

    __send_request(FAL_EVENTLOG_PATH, entry)


def handle_virus_infection_event(notification_node):
    """Handle "VirusInfection" event."""

    result_str = notification_node.get_value(SCAN_ACTION_RESULT_NAME) 
    if not result_str:
        return

    target_path = __get_target_path(notification_node)
    share = __get_share_name(target_path)
    target_file = os.path.basename(target_path)
    path_in_archive = __try_get_path_in_archive(notification_node)
    if path_in_archive:
        target_file += " (%s)" % (path_in_archive)

    if result_str.find(RESULT_DELETED) >= 0:
        __create_notify_file(target_path, NOTIFY_FILE_DELETED_PREFIX)
        log_eventlog(Msg.ERR_TMNAS_VIRUS_DELETED,
                        {SHARE_NAME: share, FILE_NAME: target_file})
        return

    if result_str.find(RESULT_QUARANTINED) >= 0:
        __create_notify_file(target_path, NOTIFY_FILE_QUARANTINED_PREFIX)
        log_eventlog(Msg.ERR_TMNAS_VIRUS_QUARANTINED,
                        {SHARE_NAME: share, FILE_NAME: target_file})
        return

    if result_str.find(RESULT_CLEANED) >= 0:
        log_eventlog(Msg.ERR_TMNAS_VIRUS_CLEAN,
                        {SHARE_NAME: share, FILE_NAME: target_file})
        return

    if result_str.find(RESULT_RENAMED) >= 0:
        log_eventlog(Msg.ERR_TMNAS_VIRUS_RENAMED,
                        {SHARE_NAME: share, FILE_NAME: target_file})
        return

    if result_str.find(RESULT_PASSED) >= 0:
        log_eventlog(Msg.ERR_TMNAS_VIRUS_PASSED,
                        {SHARE_NAME: share, FILE_NAME: target_file})
        return

    if not os.path.exists(target_path):
        __create_notify_file(target_path, NOTIFY_FILE_UNHANDLED_PREFIX)
    log_eventlog(Msg.ERR_TMNAS_VIRUS_FAIL_CLEAN,
                    {SHARE_NAME: share, FILE_NAME: target_file})
    return


#
# Main
#

# Force fd 1 is opened
null_fd = os.open(os.devnull, os.O_WRONLY)
if null_fd != 1:
    os.dup2(null_fd, 1)
    os.close(null_fd)

reset_signal()

for event_name in sys.argv[1:]:
    try:
        event_file = os.path.join(os.environ[SPOOL_PATH_ENV], event_name)

        if not os.path.exists(event_file):
            continue

        event_xml = xmldata.XmlData(event_file)

        notification_node = event_xml.get_node(NOTIFICATION_NAME)
        if notification_node.get_attr_value(TYPE_ATTR) == VIRUSINFECTION_TYPE:
            handle_virus_infection_event(notification_node)
            error_buzzer()

        os.unlink(event_file)

    except:
        # Ignore error
        pass
